@@ -41,21 +41,29 @@ try:
     config.AVAILABLE_XARRAY = True
 except:
     config.AVAILABLE_XARRAY = False   
-    
-def to_XDataArray(tableSet, dimensions = ['region', 'year', 'pathway']):
-    #%%
-#    dimensions = ['region', 'year', 'scenario', 'model']
-    
+
+
+def get_dimension_extend(table_iterable, dimensions):
+    """
+    This functions assesses the the unique extend for various dimensions
+    given a set of datatables
+    """
     fullIdx = dict()
     for dim in dimensions:
         fullIdx[dim] = set()
 
-    for table in tableSet:
+    for table in table_iterable:
         
+#        for metaKey, metaValue in table.meta.items():
+#            if metaKey not in metaDict.keys():
+#                metaDict[metaKey] = set([metaValue])
+#            else:
+#                metaDict[metaKey].add(metaValue)
+            
         for dim in dimensions:
             if dim == 'region':
                 fullIdx[dim] = fullIdx[dim].union(table.index)
-            elif dim == 'year':
+            elif dim == 'time':
                 fullIdx[dim] = fullIdx[dim].union(table.columns)
             elif dim in table.meta.keys():
                 fullIdx[dim].add(table.meta[dim])
@@ -64,17 +72,58 @@ def to_XDataArray(tableSet, dimensions = ['region', 'year', 'pathway']):
 
     dimSize = [len(fullIdx[x]) for x in dimensions]
     dimList = [sorted(list(fullIdx[x])) for x in dimensions]
-    xData =  xr.DataArray(np.zeros(dimSize)*np.nan, coords=dimList, dims=dimensions)
     
+    return dimSize, dimList
+    
+
+def get_meta_collection(table_iterable, dimensions):
+
     
     metaCollection = dict()
+    for table in table_iterable:
+        
+        for key in table.meta.keys():
+            if key in dimensions  or key == 'ID':
+                continue
+            if key not in metaCollection.keys():
+                metaCollection[key] = set()
+                
+            metaCollection[key].add(table.meta[key])
+    
+    return metaCollection
+
+def to_XDataSet(tableSet, dimensions):
+    
+    dimSize, dimList = get_dimension_extend(tableSet, dimensions= ['region', 'time'])
+    
+    dimensions= ['region', 'time']
+    
+    dSet = xr.Dataset(coords = {key: val for (key, val) in zip(dimensions, dimList)})
+    
+    for key, table in tableSet.items():
+        dSet[key] = table
+        dSet[key].attrs = table.meta
+        
+    return dSet
+    
+def to_XDataArray(tableSet, dimensions = ['region', 'time', 'pathway']):
+    #%%
+#    dimensions = ['region', 'time', 'scenario', 'model']
+    
+#    metaDict = dict()
+    
+    dimSize, dimList = get_dimension_extend(tableSet, dimensions)
+    metaCollection = get_meta_collection(tableSet, dimensions)
+     
+    xData =  xr.DataArray(np.zeros(dimSize)*np.nan, coords=dimList, dims=dimensions)
+    
     for table in tableSet:
         
         indexTuple = list()
         for dim in dimensions:
             if dim == 'region':
                 indexTuple.append(list(table.index))
-            elif dim == 'year':
+            elif dim == 'time':
                 indexTuple.append(list(table.columns))
             else:
                 indexTuple.append(table.meta[dim])
@@ -82,15 +131,17 @@ def to_XDataArray(tableSet, dimensions = ['region', 'year', 'pathway']):
 #        xx = (table.index,table.columns,table.meta['pathway'])
         xData.loc[tuple(indexTuple)] = table.values
         
-        for key in table.meta.keys():
-            if key not in metaCollection.keys():
-                metaCollection[key] = set()
-                
-            metaCollection[key] = metaCollection[key].union([table.meta[key]])
-    
+        
     # only implemented for homgeneous physical units
     assert len(metaCollection['unit']) == 1
     xData.attrs['unit'] = list(metaCollection['unit'])[0]   
+    
+    for  metaKey, metaValue in metaCollection.items():
+        if len(metaValue) == 1:
+            xData.attrs[metaKey] = metaValue.pop()
+        else:
+            print('Warning, dropping meta data: ' + metaKey +  ' ' + str(metaValue))
+    
     return xData
 #%%
 
@@ -200,8 +251,10 @@ def _AR4_conversionFactor(unitFrom, unitTo):
     assert len(co2eqkeys) == 1 and len(gasesToconvert) == 1
     co2Key = co2eqkeys[0]
     gasKey = gasesToconvert[0]
-    print(co2Key)
-    print(gasKey)
+        
+    if config.DEBUG:    
+        print('Converting from {} to {} using GWP AR4'.format(gasKey,co2Key))
+        
     
     unitFrom = unitFrom.replace(gasKey, co2Key)
     conversFactor = getUnit(unitFrom).to(unitTo).m
